/*
 * This file is part of the SOAFaces Project (hereinafter "SOAFaces").
 * Copyright (C) 2006 Grand Logic, Inc.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package org.soafaces.bundle.client.ui;

import com.google.gwt.core.client.EntryPoint;
import com.google.gwt.user.client.ui.Panel;
import org.soafaces.bundle.client.rpc.WebletContext;
import org.soafaces.bundle.client.rpc.SuccessFailCallback;
import org.soafaces.bundle.client.rpc.impl.ApplicationBoot;


/**
 * <p>
 * This class should be subclassed to create a custom editor (Weblet) for a
 * given Tasklet or to create a standalone application. This UI is considered
 * a "design time" UI for customizing the Tasklets input JavaBean.
 * </p>
 * <p>
 * When used with a Tasklet to build a workflow/job, the Weblet
 * allows the user the ability to manage a Tasklet's
 * input JavaBean (if it has input JavaBean). If the Tasklet has
 * an input JavaBean then the Weblet can read and write to this JavaBean.
 * This class functions as the top level GUI container for
 * all GUI components used in the Weblet. Note, that implementing
 * this class is optional. If a Tasklet has no need for a GUI
 * Weblet, where a user can maunally edit/view properties, then there
 * is no need to implement this class. 
 * </p>
 * <p>
 * Similarly when used with a standalone application (to build an application vs a workfow)
 * the Weblet is used to house all the GUI components for the application. And
 * optionally the application can use an input JavaBean to persist some or all of its
 * data. Check <code>WebletContext</code> for more details on using a JavaBean
 * for persistence. In general the Weblet can use a any JavaBean that implements
 * <code>IsSerializable</code> or any object that can be serialized into XML using
 * XMLStream. Simply define the input JavaBean in the
 * SFB's MANIFEST.MF and it will automically be persisted for you - that easy!
 * </p>
 *
 * @author Sam Taha
 *
 */
public abstract class Weblet implements EntryPoint {

  /**
   * Weblet can be in one of three modes.
   * 
   * EDIT - User can edit and make any changes to the input JavaBean
   *        but with limited control over workflow configuration.
   * 
   * READONLY - Not allowed to save input JavaBean 
   * 
   * DESIGN - Mode when designing the application or workflow and eding input JavaBean.
   * 
   * DEMO - Very limited to test GUI functionality primarly
   */
  public enum Mode {EDIT, READONLY, DESIGN, DEMO}

	private Mode _iMode=Mode.EDIT;
  private WebletContext _oContext=null;

	protected Weblet() {}
  
  /**
   * Initialize the Weblet including creating/populating the UI.
   * 
   * @param context
	 * @param mode Allows the Weblet to work under different modes. For example,
	 *             <code>DEMO</code> mode typically applies when the Weblet is used to
   *             demonstrate how the UI works. <code>VIEWONLY</code> prevents the user
	 *             from using the GUI to make changes or edits. This can be useful
   *             when viewing historically saved input JavaBeans for example or if the 
   *             user has limited rights.
   */
	final public void initUI(WebletContext context, Mode mode)
	{
    if(_oContext != null)
      return;
		_iMode=mode;
    _oContext=context;
    buildUI();
	}
  
  /**
   * This can return <code>RootPanel.get()</code> or any other panel you want to house your UI.
   *
   */
  public abstract Panel getMainPanel();
  
  /**
   * Implement this to create all the widgets that will be housed in this
   * top level container. This is called only once to populte the UI.
   * 
   */
  protected abstract void buildUI();

  final public WebletContext getContext() { return _oContext; }
  
  /**
	 * Override this method to save changes to the local
   * input JavaBean. This method is also used to test 
	 * validation of the GUI state before saving changes to the input JavaBean. 
	 * This allows the Weblet to enforce its own validation
	 * rules on the input JavaBean. The outside container calls this method
   * before saving the JavaBean to the server. Do not call this method
   * directly.
	 * 
   * @param callback Used in this method to relay when the outside container
   *                 can safely save the input JavaBean. This is need in case
   *                 this method needs to perform an asynchronous call before
   *                 allowing the save to occur. Make sure to that <code>callback.returnSuccess()</code>
   *                 or <code>callback.return.Failure()</code> are returned. It is highly
   *                 recommended to place this methods implementation in a try/catch/finally
   *                 to insure that returnSuccess or returnFailure is called.
   */
	public void onSaveInputBean(SuccessFailCallback callback)
  {
    callback.returnSuccess();
  }
  
	/**
	 * Only the more advanced and complex implementations need to concern
	 * themselves with what <code>mode</code> the Weblet is in. The average
	 * Weblet will typically not care about this state.
	 * 
	 * A subclass of the specific Weblet can use this property
	 * to control how the GUI functions.
	 * It is up to the Weblet to use this property to determine
	 * how to limit its functionality  \of the Weblet.
	 * 
	 * For example, if your Weblet connects to a LDAP server
	 * to pull up a list of users to select from, this LDAP connection
	 * may not be necessary when the Weblet is in <code>VIEWONLY</code>
	 * mode and in <code>DEMO</code> mode. This allows the Weblet
	 * to adapt its functionality based on what situation it has been
	 *
	 * The possible modes are:
	 *
	 *
	 * EDIT_MODE mode<br>
	 * The Weblet should function fully in this mode. User can edit and
	 * make any changes to the input JavaBean and any other properties
	 * such as data stored in the sandbox directory.
	 * <br><br>
	 * 
	 * READONLY_MODE mode<br>
	 * Limits what the Weblet can do when in a read only mode (not allowed to save input JavaBean).
	 * The Weblet will only allow user to view the input JavaBean
	 * and other resources. User is not allowed to make any changes.
	 * <br><br>
   * 
   * DESIGN_MODE mode<br>
	 * Limits what the Weblet can do when in a design mode.
	 * The Weblet will only allow user to view the input JavaBean
	 * and other resources. User is not allowed to make any changes.
	 * <br><br>
	 *
	 * DEMO_MODE mode<br>
	 * Limits the Weblet's functionality even further.
	 * This mode allows the UI to be rendered to show the user the basic
	 * capabilities of the UI. A default input JavaBean is passed in
	 * and the user should not be able to access or edit any resources.
	 *
	 * @return The mode the Custmizer is in.
	 */
	final public Mode getMode()
	{
		return _iMode;
	}
  
  final public void onModuleLoad() {
    ApplicationBoot app = new ApplicationBoot();
    app.createApplication(this);
  }
}
